#!/usr/bin/env python

''' 

This program collects statistics of disjuncts. 
This statistics are used in deciding the dimension 
of the vector space used in clustering disjuncts.

'''

import psycopg, sys, re

#-----------------------------------------------------------------
# Database Deatails
dbName= "lgstats"
userName= "siva"	
# Most systems have "postgres" as user name

connection = psycopg.connect('dbname=%s' %(dbName), 'user=%s' %(userName))
mark = connection.cursor()

#---------------------------------------------------------------------
# Get all the valid disjuncts
# Assumption: The disjunct of a word is eligible to 
# consider only when its occurance with that word is 
# more than a threshhold

min_observed_weight= 3.0
command= "SELECT disjunct, count from public.disjuncts where count >= %f" %(min_observed_weight)
mark.execute(command)
disjuncts= mark.fetchall()



def generalize_a_disjunct(disjunct):
	'''Generalizes the given disjunct 
	For eg: S- Os+ MVp+ MVp+ --> @S- @O- @MV+'''
	final_link= ""
	links= disjunct.split()
	prev_link= ""
	for link in links:
		link= re.sub("[a-z\*]", "", link)
		link= "@" + link
		if link==prev_link:
			continue
		else:
			final_link += link + " "
		prev_link= link
	return final_link.strip();

#--------------------------------------------------------------------
# Get the statistics

def get_statistics(type=0):
	''' Type 0 = Get the counts of disjuncts without 
		modifying the structure of disjunct 
	    Type 1= Get the counts of disjuncts with
	        additional attributes of disjuncts removed
	    Type 2= Type 1 + regular expressions '''

	counts= {}	# word frequency: number of times 
	# the disjunct occured with distinct words. Its 
	# structure is { disjunt:count }
	
	for (disjunct, count) in disjuncts:
		disjunct= disjunct.strip()
		if type==2:
			disjunct= generalize_a_disjunct(disjunct)
		elif type==1:
			disjunct= re.sub("[a-z\*]", "", disjunct)
		#print disjunct
		if not counts.has_key(disjunct):
			counts[disjunct]= [0, 0.0]
		counts[disjunct][0] += 1
		counts[disjunct][1] += count
	return counts

def compare(item1, item2):
    '''Used in sorting disjuncts based on their scores'''
    if item1[1][0] < item2[1][0]:
        return 1;
    elif item1[1][0] > item2[1][0]:
        return -1;
    else:
            if item1[1][1] < item2[1][1]:
                return 1
            elif item1[1][1] > item2[1][1]:
                return -1
            else:
                return 0

if __name__=="__main__":
	''' Takes command line arguments
	Type 0 = Get the counts of disjuncts without 
	         modifying the structure of disjunct 
	Type 1= Get the counts of disjuncts with
	        additional attributes of disjuncts removed
	Type 2= Type 1 + regular expressions '''

	counts= get_statistics(int(sys.argv[1]))
	items= counts.items()
	items.sort(compare)
	for item in items:
		print "%s\t%s\t%s" %(item[0], item[1][0], item[1][1])
	print "Total Disjuncts", len(items)
